//AT+CWJAP_DEF="ESP8266","0123456789"

#include <SoftwareSerial.h>
#include <avr/wdt.h>

#define ESP_RESET  A2

SoftwareSerial wifiSerial(7, 8);      // RX, TX for ESP8266

String AP = "ESP8266"; // CHANGE ME
String PASS = "1234567890"; // CHANGE ME
String cwSAP;

void(* resetFunc) (void) = 0;//declare reset function at address 0

bool DEBUG = true;   //show more logs
int responseTime = 10000; //communication timeout

String responses;
String channel;

void setup()
{
  pinMode(13, OUTPUT); //set build in led as output
  pinMode(ESP_RESET, OUTPUT); //set build in led as output

  //Resetovanje ESP_MODULA
  digitalWrite(ESP_RESET, LOW);
  delay(500);
  digitalWrite(ESP_RESET, HIGH);
  delay(500);
  

  // Open serial communications and wait for port to open esp8266:
  Serial.begin(9600);
  wifiSerial.begin(9600);

  if (sendToWifi("AT", responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // inicijalizacija esp modula
  if (sendToWifi("AT+CWMODE=2", responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // configure as access point
  cwSAP = "AT+CWSAP_DEF=\"" + AP + "\",\"" + PASS + "\",5,3";
  if (sendToWifi(cwSAP, responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // configures the ESP8266 SoftAP; configuration Saved in the Flash  
  if (sendToWifi("AT+CIFSR", responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // get ip address
  if (sendToWifi("AT+CIPMUX=1", responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // configure for multiple connections
  if (sendToWifi("AT+CIPSERVER=1,80", responseTime, DEBUG, "OK", responses) != 0)resetFunc(); // turn on server on port 80

  Serial.println(F("Wifi connection is running!"));

  wdt_enable(WDTO_8S);  
}


void loop()
{
  if (wifiSerial.available() > 0) {
    String message = readWifiMessage();
    Serial.println(message);

    if(find(message, "CONNECT")){
      channel = message.substring(0, 1);
      Serial.println(channel);
    }
    else if (find(message, "HELLO")) { //receives HELLO from wifi
      channel = message.substring(5, 6);
      sendData(channel, "\nHI!");    //arduino says HI
    }
    else if (find(message, "Kako ide?")) { //receives HELLO from wifi
      channel = message.substring(5, 6);
      sendData(channel, "\nEvo po malo. :)");    
    }    
    else if (find(message, "LEDON")) {
      digitalWrite(13, HIGH);
      sendData(channel, "\nUkljucena LED!"); 
    }
    else if (find(message, "LEDOFF")) {
      digitalWrite(13, LOW);
      sendData(channel, "\nIskljucena LED!"); 
    }
    else if(message.length() > 0){
      channel = message.substring(5, 6);
      sendData(channel, "\nNe lupaj!! :(");                 //Command ERROR CODE for UNABLE TO READ
    }
  }
  
  wdt_reset();
  delay(1);
}


/*
  Name: sendData
  Description: Function used to send string to tcp client using cipsend
  Params:
  Returns: void
*/
int sendData(String chn, String str) {
  String len = "";
  
  len=String(str.length()+1);
  Serial.print("AT+CIPSEND=" + chn + "," + len);
  
  if (sendToWifi("AT+CIPSEND=" + chn + "," + len, responseTime, DEBUG, ">", responses)!= 0)resetFunc();
  if (sendToWifi(str, responseTime, DEBUG, "SEND OK", responses) != 0)resetFunc();

  return(0);
}


/*
  Name: find
  Description: Function used to match two string
  Params:
  Returns: true if match else false
*/
boolean find(String string, String value) {
  if (string.indexOf(value) >= 0)
    return true;
  return false;
}


/*
  Name: readWifiSerialMessage
  Description: Function used to read data from ESP8266 Serial.
  Params:
  Returns: The response from the esp8266 (if there is a reponse)
*/
String  readWifiMessage() {
  String value;
  if (wifiSerial.available()) {
    value+=wifiSerial.readString();
  }
  value.trim();
  return value;
}



/*
  Name: sendToWifi
  Description: Function used to send data to ESP8266.
  Params: command - the data/command to send; timeout - the time to wait for a response; debug - print to Serial window?(true = yes, false = no)
  Returns: The response from the esp8266 (if there is a reponse)
*/
int sendToWifi(String command, const int timeout, boolean debug, String rsp, String response) {
  response = "";
  int rez = 1;
  wifiSerial.println(command); // send the read character to the esp8266
  long int time = millis();
  while ( (time + timeout) > millis())
  {
    if (wifiSerial.available())
    {
      // The esp has data so read it
      response+=wifiSerial.readString();
    }
    if (rsp != "")
    {
      if (find(response, rsp)) {
        rez = 0;
        break;
      }
      else if (find(response, "no change")) {
        rez = 0;
        break;
      }
      else if (find(response, "link is builded")) {
        rez = 0;
        break;
      }
    }
    else {
      rez = 0;
      break;
    }
    wdt_reset();
  }
  if (debug)
  {
    Serial.println(response);
  }
  return rez;
}
